from .models import (
    Plant,
    EcologicalRegion,
    EcologicalDistrictLayer,
    ChristchurchRegion,
    SoilOrder,
    SoilVariant,
    ActivationKey,
    Questionnaire
)
from .wms_utils import get_point_from_coordinates


def is_in_auckland(coordinates):
    pnt = get_point_from_coordinates(coordinates)
    eco_district = EcologicalDistrictLayer.objects.filter(geom__intersects=pnt).first()
    print(eco_district.ecologic_2)
    return eco_district is not None and eco_district.ecologic_2.name == 'Auckland'


def is_in_christchurch(coordinates):
    pnt = get_point_from_coordinates(coordinates)
    in_chch = ChristchurchRegion.objects.filter(geom__intersects=pnt).first()
    return in_chch is not None


def get_filtered_plants(request):
    try:
        ak = ActivationKey.objects.get(key=request.query_params['key'])
    except (KeyError, ActivationKey.DoesNotExist):
        raise ValueError("Invalid key")

    if not (q := Questionnaire.objects.filter(key=ak).order_by("-creation_date").first()):
        return Plant.objects.none()

    regions = EcologicalRegion.objects.filter(ecologicaldistrictlayer__geom__intersects=q.location).distinct()
    soils = (
        SoilVariant.objects.filter(name="Mesic") |
        SoilVariant.objects.filter(name__istartswith=q.soil_variant)
    ).distinct()

    qs = Plant.objects.filter(
        zones__in=[q.zone],
        ecological_regions__in=regions,
        soil_variants__in=soils,
    )

    if not q.zone.ignore_soil_order_filter:
        orders = SoilOrder.objects.filter(soillayer__geom__intersects=q.location).distinct()
        qs = qs.filter(soil_order__in=orders).distinct()

    return qs.distinct()
